import sys
import math
import OrcFxAPI

class thrust(object):

    # Setting initial values. 'Initialise' is only run once.
    def Initialise(self, info):
    
        # Reading parameters from OrcaDlex data form:
        paramsDict = {}
        for key, value in info.ObjectParameters.items():
            paramsDict[key] = float(value)
        self.params = OrcFxAPI.objectFromDict(paramsDict)
        
        # Collecting the time-step from the OrcaFlex model
        self.timeStep = info.Model.general.ImplicitConstantTimeStep
        
        # There is one instance of this external function per data item that uses it. 
        # As long as the applied load calculations are uncoupled, each instance will keep 
        # track of the required data in the attributes of each instance.
        self.int = 0.0
        self.intLastIteration = 0.0
        if info.DataName.startswith('GlobalAppliedForceX'):
            self.prevpos = self.params.target1
            self.prevposLastIteration = self.params.target1
            self.target = self.params.target1
            self.kp = self.params.kp1
            self.ki = self.params.ki1
            self.kd = self.params.kd1
            self.kd = self.params.kd1
            self.stat = self.params.stat1
        if info.DataName.startswith('GlobalAppliedForceY'):
            self.prevpos = self.params.target2 
            self.target = self.params.target2
            self.kp = self.params.kp2
            self.ki = self.params.ki2
            self.kd = self.params.kd2
            self.prevposLastIteration = self.params.target2
            self.stat = self.params.stat2
        if info.DataName.startswith('GlobalAppliedMomentZ'):
            self.prevpos = self.params.target6 
            self.prevposLastIteration = self.params.target6          
            self.target = self.params.target6
            self.kp = self.params.kp6
            self.ki = self.params.ki6
            self.kd = self.params.kd6
            self.stat = self.params.stat6
            
    def Calculate(self, info):
        if info.DataName.startswith('GlobalAppliedForceX'):
            actual = info.InstantaneousCalculationData.Position[0]
        elif info.DataName.startswith('GlobalAppliedForceY'):
            actual = info.InstantaneousCalculationData.Position[1]
        elif info.DataName.startswith('GlobalAppliedMomentZ'):
            Xaxis = info.InstantaneousCalculationData.Orientation[0]
            actual = math.degrees(math.atan2(Xaxis[1], Xaxis[0]))
        else:
            raise Exception('Cannot find name of value to be determined')

        #If it's a new time step then update prevpos and int to be the values from the previous iteration of the implicit solver,
        #if it's the first time step then they will take the inital values set in Initialise:
        if info.NewTimeStep:
            self.prevpos = self.prevposLastIteration
            self.int = self.intLastIteration

        error = actual-self.target
        Fp = -self.kp*error
        Fi = self.int - self.ki*error*self.timeStep
        Fd = -self.kd*(actual-self.prevpos)/self.timeStep 

        #Store data from this iteration ready to be used if we go to a new time step:
        self.prevposLastIteration = actual 
        self.intLastIteration = Fi

        info.Value = Fp+Fi+Fd+self.stat
